;;; ENV-EVAL.SCM
;;; Evaluation of INTEX programs using environments

;; (env-run <pgm> <intlist>) 
;;   Evaluates the given program on inputs <intlist>
;;   using the environment model.
(define env-run
  (lambda (pgm ints)
    (env-eval (program-body pgm)
              (env-make (program-formals pgm) ints))))

;; (env-eval <exp> <env>) 
;;   Evaluate the expression <exp> relative to the given 
;;   environment <env>, which determines the meaning of
;;   variable references appearing in <env>.
(define env-eval
  (lambda (exp env)
    (cond ((literal? exp) 
           (literal-value exp))
          ((varref? exp) 
           (let ((probe (env-lookup (varref-name exp) env)))
             (if (unbound? probe)
               (throw 'unbound-variable (varref-name exp))
               probe)))
          ((binapp? exp)
           ((binop-to-function (binapp-rator exp))
            (env-eval (binapp-rand1 exp) env)
            (env-eval (binapp-rand2 exp) env)))
	  (else ("ENV-EVAL: Unknown expression type -- " exp))
          )))

;; Note: BINOP-TO-FUNCTION is provided by SUBST-EVAL.SCM
