;;; Examples of INTEX programs using s-expression syntax

;; Incrementing function
(define inc
  '(program (x) (+ x 1)))

;; Centigrade to Fahrenheit
(define c2f
  '(program (cent)
     (+ 32
        (div (* 9 cent)
             5))))

;; Fahrenheit to Centigrade
(define f2c
  '(program (fahr)
     (div (* 5 (- fahr 32))
          9)))

;; Average of two numberss
(define avg
  '(program (a b)
     (div (+ a b) 2)))

(define ratio
  '(program (a b)
     (div (+ (* a a) (* b b))
	  (- (* a a) (* b b)))))

(define test-unbound
  '(program (a b)
      (* (+ a b) c)))

(define test-div-by-zero
  '(program (a b c)
      (div (+ a b) c)))

(define test-mod-by-zero
  '(program (a b c)
      (mod (+ a b) c)))

(define test-suite
  (list
   (list 'inc inc '(3) 4)
   (list 'c2f c2f '(100) 212)
   (list 'c2f c2f '(0) 32)
   (list 'c2f c2f '(-40) -40)
   (list 'f2c f2c '(212) 100)
   (list 'f2c f2c '(32) 0)
   (list 'f2c f2c '(-40) -40)
   (list 'ratio ratio '(10 8) 4)
   (list 'test-unbound test-unbound '(1 2)
	 '(bindex-error unbound-variable c))
   (list 'test-div-by-0 test-div-by-zero '(2 1 0)
	 '(bindex-error division-by-zero (div 3 0)))
   (list 'test-mod-by-0 test-mod-by-zero '(2 1 0)
	 '(bindex-error division-by-zero (mod 3 0)))
   ))

;; Tests a program execution function, such as SUBST-RUN or ENV-RUN
(define test-run
  (lambda (run)
    (begin
      (for-each (run-test-case run) test-suite)
      (newline)
      (display "Done.")
      (newline))))

(define run-test-case
  (lambda (run)
    (lambda (test-case)
      (let ((program-name (first test-case))
            (program (second test-case))
            (inputs (third test-case))
            (expected-result (fourth test-case)))
        (begin
          (newline)
          (display "Running ")
          (display program-name)
          (display " on ")
          (display inputs)
          (display " gives ")
          (let ((result (catch (lambda () (run program inputs))
			       (lambda (tag value) (list 'bindex-error tag value)))))
            (if (equal? result expected-result)
              (begin (display result)
                     (display ". OK!"))
              (begin (display result)
                     (display " ***ERROR!*** Expected ")
                     (display expected-result)))))))))
                  

