;;; Evaluation of INTEX programs using substitution

;; (subst-run <pgm> <intlist>) 
;;   Evaluates the given program on inputs <intlist> 
;;   using the substitution model.
(define subst-run
  (lambda (pgm ints)
    (literal-value
     (subst-eval (subst* (map make-literal ints)
			 (program-formals pgm)
			 (program-body pgm))))))

;; (subst-eval <exp>) 
;;   Evaluate the expression <exp>, assuming that integer
;;   values have already been substituted for all variable
;;   references. It is an error if the substitution evaluator
;;   encounters any variables; these muse be unbound.
;;   Note that SUBST-EVAL returns an integer literal node, 
;;   *not* an integer.
(define subst-eval
  (lambda (exp)
    (cond ((literal? exp) exp)
          ((varref? exp) 
           (throw 'unbound-variable (varref-name exp)))
          ((binapp? exp)
           ((binop-to-function (binapp-rator exp))
            (subst-eval (binapp-rand1 exp))
            (subst-eval (binapp-rand2 exp))))
          (else (error "SUBST-EVAL: Unknown expression type -- " exp))
          )))

;; Returns a Scheme function that takes to integer
;; literal nodes and returns a integer literal node
;; whose value is the results of applying the 
;; specified operator to the values of the two arguments.
(define binop-to-node-function
  (lambda (binop)
    (lambda (lit1 lit2)
      (make-literal 
        ((binop-to-function binop)
         (literal-value lit1)
         (literal-value lit2))))))

;; Returns a Scheme int-to-int function indicated by the symbol
;; Given the symbol name for a binary operator, 
;; return the corresponding Scheme procedure.
(define binop-to-function
  (lambda (binop)
    (cond ((eq? binop '+) +)
          ((eq? binop '-) -)
          ((eq? binop '*) *)
          ((eq? binop 'div) 
           (lambda (a b)
             (if (= b 0)
                 (throw 'division-by-zero (list 'div a b))
                 (quotient a b))))
          ((eq? binop 'mod) 
           (lambda (a b)
             (if (= b 0)
                 (throw 'division-by-zero (list 'mod a b))
                 (remainder a b))))
          (else (error "BINOP-TO-FUNCTION: unrecognized binop " binop))
          )))

