;;; SUBST.SCM
;;; Performing substitution on INTEX expressions.

;; (subst <env> <exp>) returns a copy of expression <exp> 
;; in which every variable reference to a name bound in <env>
;; is replaced by the expression bound to the name. When
;; using SUBST in SUBST-EVAL, the entity bound to the name 
;; should be an integer literal node, *not* an integer. 
(define subst
  (lambda (env exp)
    (cond ((literal? exp) exp)
          ((varref? exp) 
	   (let ((probe (env-lookup (varref-name exp) env)))
	     (if (unbound? probe) exp probe)))
          ((binapp? exp)
           (make-binapp (binapp-rator exp)
                        (subst env (binapp-rand1 exp))
                        (subst env (binapp-rand2 exp))))
          (else (error "SUBST: Unrecognized expression -- " exp))
          )))

;; Auxiliary functions for substitution 
;; Note that the order of replacement expressions ("new" 
;; and "news") and variables ("old" and "olds") in subst1 
;; and subst* is different from the order in which they
;; are supplied to env-make
(define subst1
  (lambda (new old exp)
    (subst (env-make (list old) (list new)) exp)))

(define subst*
  (lambda (news olds exp)
    (subst (env-make olds news) exp)))


